from collections import deque, defaultdict, Counter
from functools import lru_cache, reduce
from math import ceil, gcd, factorial
from itertools import chain, product, accumulate, combinations
from heapq import heappop, heappush, heapify
from bisect import bisect_left, bisect_right
from string import digits, ascii_lowercase
from math import inf